#include "opener.h"

/* Support for providing access to files. */

Opener::Opener(Accessing *accessing, Paging *paging, Pages *pages)
: _accessing(accessing), _paging(paging), _pages(pages)
{
}

OpenerContext *Opener::context()
{
    return new OpenerContext(this);
}

FilePager *Opener::open(const char *path)
{
    fileid_t fileid = get_fileid(path);

    if (fileid == FILEID_INVALID)
        return NULL;

    Accessor *accessor = get_accessor(fileid);
    PageMapper *mapper = get_mapper(accessor);
    return new FilePager(fileid, mapper);
}

/* Obtain an accessor for the 'fileid' or register a new one in the accessing
   object. */

Accessor *Opener::get_accessor(fileid_t fileid)
{
    Accessor *accessor;

    if (_accessing->reserve(fileid))
        accessor = _accessing->get(fileid);
    else
    {
        accessor = make_accessor(fileid);
        _accessing->set(fileid, accessor);
    }

    return accessor;
}

/* Obtain a page mapper for the 'accessor' or register a new one in the
   paging object. */

PageMapper *Opener::get_mapper(Accessor *accessor)
{
    PageMapper *mapper;

    if (_paging->reserve(accessor))
        mapper = _paging->get(accessor);
    else
    {
        mapper = new PageMapper(accessor, _pages);
        _paging->set(accessor, mapper);
    }

    return mapper;
}

// vim: tabstop=4 expandtab shiftwidth=4
